(* ::Package:: *)

(*Package containing functions that compute symbolically and numerically the general solution of the Fredholm integral equation of the second kind. 
a) with a product kernel 
b) using the Neumann series for an arbitrary kernel
c) using a numerical version of the Neumann series*)
  (*M. Wenin 2014*)
(*Bibliography: Bronstein Semendjajew, Erg\[ADoubleDot]nzende Kapitel zum Taschenbuch der Mathematik, herausgegeben von G. Grosche, V. Ziegler und D. Ziegler, 6. Auflage, 
   Teubner Verlagsgesellschaft Leibzig, 1990.
 Weisstein, Eric W. "Integral Equation." From MathWorld--A Wolfram Web Resource. http://mathworld.wolfram.com/IntegralEquation.html*) 

BeginPackage["integralequationfredholmpackage`"];
 
integraleqfredholmhom::usage =
        "integraleqfredholmhom[A_List,B_List,x_Symbol,{y_Symbol,a_,b_}] solves symbolically the homogeneous Fredholm Integral equation of the second kind,
         Integrate[A[x].B[y] * phi[y],{y,a,b}]= phi[x]
         with a product kernel written as A[x].B[y]. A, B are lists of equal length, a,b are the integration limits.";

integraleqfredholm::usage =
        "integraleqfredholm[A_List,B_List,f_,x_Symbol,{y_Symbol,a_,b_}] solves symbolically the Fredholm Integral equation of the second kind 
         with a product kernel written as A[x].B[y]. A, B are lists of equal length. f[x] is the inhomogeneous part, a,b are the integration limits. 
         Integrate[A[x].B[y] * phi[y],{y,a,b}] + f[x] = phi[x]";

integraleqneumannseries::usage=
         "integraleqneumannseries[K_,{f_,f0_},x_Symbol,{y_Symbol,a_,b_},n_] computes symbolically the Neumann series for the Fredholm integral equation 
          Integrate[K[x,y] * phi[y],{y,a,b}] + f[x] = phi[x] for an arbitrary Kernel K[x,y].
          f0[x] is the start function with default f0[x]=0, n is the order of the series."; 

nintegraleqneumannseries::usage=
                                "nintegraleqneumannseries2[K_,f_,{y_Symbol,a_,b_},{x_Symbol,X0_,X1_},n1_?IntegerQ/;n1>0,n2_?IntegerQ/;n2>0,intord_:3,
                                 printnorm_:0,opts:OptionsPattern[]]
                                 computes numerically the Neumann series for the equation Integrate[K[x,y] * phi[y],{y,a,b}] + f[x] = phi[x] up to the order n1. 
                                 The iteration starts with phi[x]=f[x] as zeroth order. 
                                 The Integer n2 is the number of grid points to discretize the interval [X0,X1]. 
                                 intord gives the order of the interpolation polynomial used to interpolate the solution. printnorm can be: 
                                 0, no information is displayed, 
                                 1 iteration steps are displayed,
                                 2 iteration steps and convergence information using the norm \[LeftDoubleBracketingBar]\!\(\*SubscriptBox[\(\[Phi]\), \(k\)]\)-\!\(\*SubscriptBox[\(\[Phi]\), \(k + 1\)]\)\[LeftDoubleBracketingBar] = 
                                 Sqrt[NIntegrate[(\!\(\*SubscriptBox[\(\[Phi]\), \(k\)]\)[x]-\!\(\*SubscriptBox[\(\[Phi]\), \(k + 1\)]\)[x])^2,{x,Xmin,Xmax}] are displayed. 
                                 All options of NIntegrate are available in ops (when an option is set, the arguments intord and printnorm must be set also).
                                 The result is an interpolatingfunction object. The functions K and f must be listable.";
                               



intfredholm0::argn="Lists `1` and `2` have different lengths.";



Begin["`Private`"];




(*Homogeneous equation, f=0*)

intfredholmhom0[A_List,B_List,x_Symbol,{y_Symbol,a_,b_}]:=
Module[{n=Length[A],Ay=A/.x->y},
Ctab=Integrate[KroneckerProduct[B,Ay],{y,a,b}];
Xnull = NullSpace[IdentityMatrix[n]-Ctab]//Simplify;

If[Xnull=={},Xsol=0,
Xsol = Sum[C[i] Xnull[[i]], {i, Length[Xnull]}];
A.Xsol]]

integraleqfredholmhom[A_List,B_List,x_Symbol,{y_Symbol,a_,b_}]:=
If[Length[A]==Length[B],intfredholmhom0[A,B,x,{y,a,b}],
Message[intfredholm0::argn,A,B]]

(*Inhomogeneous equation*)

intfredholm0[A_List,B_List,f_,x_,{y_Symbol,a_,b_}]:=
Module[{n=Length[A],Ay=A/.x->y,fy=f/.x->y},
ftab=Integrate[fy B,{y,a,b}];
Ctab=Integrate[KroneckerProduct[B,Ay],{y,a,b}];
Xsol=LinearSolve[IdentityMatrix[n]-Ctab,ftab]//Simplify;
f+A.Xsol]

integraleqfredholm[A_List,B_List,f_,x_Symbol,{y_Symbol,a_,b_}]:=
If[Length[A]==Length[B],intfredholm0[A,B,f,x,{y,a,b}],
Message[intfredholm0::argn,A,B]]






(*Neumann series: symbolic version*)

integraleqneumannseries[K_,{f_,f0_:0},x_Symbol,{y_Symbol,a_,b_},n_?IntegerQ/;n>0]:=
                 Module[{\[Phi]=f0},
                 Do[\[Phi] =(f+ Integrate[(K/.y->Subscript[y, j-1]) \[Phi],{Subscript[y, j-1],a,b}])/.x->Subscript[y, j],
                    {j,1,n}];\[Phi]/.Subscript[y, n]->x]
                



(*Neumann series, numerical version*)

nintegraleqneumannseries[K_,f_,{y_Symbol,a_,b_},{x_Symbol,X0_,X1_},n1_?IntegerQ/;n1>0,n2_?IntegerQ/;n2>0,intord_:3,printnorm_:0,opts:OptionsPattern[]]:=
                 Module[{dx,Xv,k,f1,f2,\[Phi]},
dx = (X1-X0)/n2;
Xv = Range[X0,X1,dx];
\[Phi] =  Interpolation[Thread[List[Xv,f[Xv]]],InterpolationOrder->intord];

k=1;
Do[f1=\[Phi];
 \[Phi] = Interpolation[Thread[List[Xv,(f[Xv] + NIntegrate[K[Xv,y]\[Phi][y],{y,a,b} ,Evaluate@FilterRules[{opts},Options[NIntegrate]]])]],InterpolationOrder->intord];

Which[printnorm==0,f2 = \[Phi],
      printnorm==1,f2 = \[Phi]; PrintTemporary["k = ",k],
      printnorm==2,f2 = \[Phi]; Print["k = ",k];
Print["\[LeftDoubleBracketingBar]\!\(\*SubscriptBox[\(\[Phi]\), \(k\)]\)-\!\(\*SubscriptBox[\(\[Phi]\), \(k + 1\)]\)\[LeftDoubleBracketingBar] = ",
Sqrt[NIntegrate[(f1[x]-f2[x])^2,{x,X0,X1},Evaluate@FilterRules[{opts},Options[NIntegrate]]]]]]; k++,{n1}];
\[Phi]/;(printnorm==0||printnorm==1||printnorm==2)];


End[ ]

EndPackage[ ]

